<?php

/**
 * Class Bot
 * @property string $token
 * @property array $data
 */
class Bot
{
    // токен бота
    public $token = "";
    // для хранения массива данных от Телеграм
    public $data;

    /**
     * Bot constructor.
     */
    public function __construct()
    {
        // записываем в свойство данные
        $this->data = json_decode(file_get_contents('php://input'), true);
    }

    /** Получаем id
     * @param $data
     * @return mixed
     */
    public function getChatId()
    {
        if ($this->getType() == "callback_query") {
            return $this->data['callback_query']['message']['chat']['id'];
        }
        return $this->data['message']['chat']['id'];
    }

    /** Получаем first_name
     * @param $data
     * @return mixed
     */
    public function getChatFirstName()
    {
        if ($this->getType() == "callback_query") {
            return $this->data['callback_query']['message']['chat']['first_name'];
        }
        return $this->data['message']['chat']['first_name'];
    }

    /** Получаем last_name
     * @param $data
     * @return mixed
     */
    public function getChatLastName()
    {
        if ($this->getType() == "callback_query") {
            return $this->data['callback_query']['message']['chat']['last_name'];
        }
        return $this->data['message']['chat']['last_name'];
    }

    /** Получаем username
     * @param $data
     * @return mixed
     */
    public function getChatUserName()
    {
        if ($this->getType() == "callback_query") {
            return $this->data['callback_query']['message']['chat']['username'];
        }
        return $this->data['message']['chat']['username'];
    }

    /** Получаем id сообщения
     * @param $data
     * @return mixed
     */
    public function getMessageId()
    {
        if ($this->getType() == "callback_query") {
            return $this->data['callback_query']['message']['message_id'];
        }
        return $this->data['message']['message_id'];
    }

    /** Получим значение текст
     * @return mixed
     */
    public function getText()
    {
        if ($this->getType() == "callback_query") {
            return $this->data['callback_query']['data'];
        }
        return $this->data['message']['text'];
    }

    /** Узнаем какой тип данных пришел
     * @param $data
     * @return bool|string
     */
    public function getType()
    {
        if (isset($this->data['callback_query'])) {
            return "callback_query";
        } elseif (isset($this->data['message']['text'])) {
            return "message";
        } else {
            return false;
        }
    }

    /** Уведомление в клиенте
     * @param $cbq_id
     * @param $text
     * @param bool $type
     */
    public function notice($text = "")
    {
        $data = [
            'callback_query_id' => $this->data['callback_query']['id'],
        ];
        if (!empty($text)) {
            $data['text'] = $text;
        }
        $this->botApiQuery("answerCallbackQuery", $data);
    }

    /** Удаляем сообщение
     * @param $chat_id
     * @param $message_id
     * @return mixed
     */
    public function deleteMessage()
    {
        return  $this->botApiQuery("deleteMessage", [
                "chat_id" => $this->getChatId(),
                "message_id" => $this->getMessageId()
            ]
        );
    }

    /** Кнопка inline
     * @param $text
     * @param string $callback_data
     * @param string $url
     * @return array
     */
    public function buildInlineKeyboardButton($text, $callback_data = '', $url = '')
    {
        // рисуем кнопке текст
        $replyMarkup = [
            'text' => $text,
        ];
        // пишем одно из обязательных дополнений кнопке
        if ($url != '') {
            $replyMarkup['url'] = $url;
        } elseif ($callback_data != '') {
            $replyMarkup['callback_data'] = $callback_data;
        }
        // возвращаем кнопку
        return $replyMarkup;
    }

    /** Набор кнопок inline
     * @param array $options
     * @return string
     */
    public function buildInlineKeyBoard(array $options)
    {
        // собираем кнопки
        $replyMarkup = [
            'inline_keyboard' => $options,
        ];
        // преобразуем в JSON объект
        $encodedMarkup = json_encode($replyMarkup, true);
        // возвращаем клавиатуру
        return $encodedMarkup;
    }

    /** Отправляем текстовое сообщение с inline кнопками
     * @param $user_id
     * @param $text
     * @param null $buttons
     * @param bool $type
     * @param bool $url
     * @return mixed
     */
    public function sendMessage($user_id, $text, $buttons = NULL)
    {
        // готовим массив данных
        $data_send = [
            'chat_id' => $user_id,
            'text' => $text,
            'parse_mode' => 'html',
        ];

        // если переданны кнопки то добавляем их к сообщению
        if (!is_null($buttons) && is_array($buttons)) {
            $data_send['reply_markup'] = $this->buildInlineKeyBoard($buttons);
        }
        // отправляем текстовое сообщение
        return $this->botApiQuery("sendMessage", $data_send);
    }

    /** Запросы в Бот АПИ
     * @param $method
     * @param array $fields
     * @return mixed
     */
    public function botApiQuery($method, $fields = array())
    {
        $ch = curl_init('https://api.telegram.org/bot' . $this->token . '/' . $method);
        curl_setopt_array($ch, array(
            CURLOPT_POST => count($fields),
            CURLOPT_POSTFIELDS => http_build_query($fields),
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_TIMEOUT => 10
        ));
        $r = json_decode(curl_exec($ch), true);
        curl_close($ch);
        return $r;
    }
}