<?php
// Подключаем класс для работы с БД
require_once("Db.php");
// подключаем класс Lang
require_once("Lang.php");
// подключаем класс User
require_once("User.php");
// подключаем класс Bot
require_once("Bot.php");

/**
 * Class WebHook
 * @property User $user
 * @property Bot $bot
 * @property Lang $lang
 */
class WebHook extends Db
{
    private $user;
    private $bot;
    private $lang;

    /**
     * Инициализируем работу класса
     */
    public function __construct()
    {
        // создаем объект бота
        $this->bot = new Bot();
        //  создаем объект пользователя
        $this->user = new User($this->prepareUserData());
        // отправляем на проверку настройки языка
        $this->userLang();
    }

    /** Роутер
     * @param $data array
     */
    private function router()
    {
        // Если это объект Message
        if ($this->bot->getType() == "message") {
            // проверяем на наличие объекта Text
            if (array_key_exists("text", $this->bot->data['message'])) {
                if ($this->bot->getText() == "/start") {
                    // Выводим приветствие
                    $this->startBot();
                } elseif ($this->bot->getText() == "/lang") {
                    // Смена настройки Lang
                    $this->changeLang();
                } else {
                    // выводим ошибку
                    $this->showError();
                }
            } else {
                // выводим ошибку
                $this->showError();
            }
        // Если это объект callback_query
        } elseif ($this->bot->getType() == "callback_query") {
            $func_param = explode("_", $this->bot->getText());
            $func = $func_param[0];
            $this->$func();
        }
        // Другие объекты не рассматриваем
        else {
            // выводим ошибку
            $this->showError();
        }
    }

    /**
     * Первый экран
     */
    private function startBot()
    {
        // выводим приветствие
        $this->bot->sendMessage($this->bot->getChatId(), $this->lang->getParam("helloText",[
            'name' => $this->user->getFullName()
        ]));
    }

    /** Готовим данные для пользователя
     * @return array
     */
    private function prepareUserData()
    {
        return [
            'telegram_id' => $this->bot->getChatId(),
            'first_name' => $this->bot->getChatFirstName(),
            'last_name' => $this->bot->getChatLastName(),
            'username' => $this->bot->getChatUserName(),
        ];
    }

    /**
     *  Выводим ошибку
     */
    private function showError() {
        $this->bot->sendMessage($this->bot->getChatId(), $this->lang->getParam("error"));
    }

    /**
     *  Проверяем выбран ли язык у пользователя
     */
    private function userLang()
    {
        if (is_null($this->user->getLang())) {
            // получаем текст
            $text = $this->bot->getText();
            // возможно это запрос на 1-ю смену языка, т.е. язык не установлен
            // просто при инлайн запросе все равно в index запрос сюда идет в userLang
            // просто перенаправляем в setUserLangInline_ а то цикл замкнутый идет
            if (preg_match("~^setUserLangInline_~", $text)) {
                // отправляем на смену
                $this->setUserLangInline();
            } else {
                // делаем запрос на смену язык
                $this->changeLang();
            }
        } else {
            // получаем языковые настройки
            $this->lang = new Lang($this->user->getLang());
            // передаем в роутер
            $this->router();
        }
    }

    /**
     * Запрос на установку языка
     */
    private function changeLang()
    {
        // готовим кнопки
        $buttons[] = [
            ['text' => '🇷🇺 Русский', 'callback_data' => 'setUserLangInline_ru'],
            ['text' => '🇬🇧 English', 'callback_data' => 'setUserLangInline_en'],
        ];
        // отправляем сообщение
        $this->bot->sendMessage($this->bot->getChatId(), 'Selected language', $buttons);
    }

    /**
     * Обработка команды inline по установке языка
     */
    private function setUserLangInline()
    {
        // 1 - lang
        $param = explode("_", $this->bot->getText());
        // устанавливаем язык пользовтаелю
        $setLang = $this->user->setLang($param[1]);
        // проверяем
        if ($setLang) {
            // глушим уведомление
            $this->bot->notice();
            // удаляем сообщение
            $this->bot->deleteMessage();
            // Определяем язык
            $this->lang = new Lang($this->user->getLang());
            // перенаправляем на старт бота
            $this->startBot();
        } else {
            // выводим сообщение об ошибке popup
            $this->bot->notice("Failed to change language, try again");
        }
    }

}